import datetime

from django.db import transaction
from djangotools.common.types import date_to_string, parse_date
from baby.app.context.base import BaseContextData, UserPrefences
from baby.app.models.regle import Regle, Cycle, Cycles


class ContextData(BaseContextData):
    page = "edit"

    def _get_cycle_data(self, i, cycle, average):
        classes = []
        classes.append("regle-doubtfull" if cycle.doubtfull else "regle-ok")
        resolved_doubtfull = cycle.resolve_doubtfull(average)

        return {
            "id" : i,
            "date" : date_to_string(cycle.start),
            "fin" : date_to_string(cycle.end),
            "length" : cycle.length,
            "classes" :  " ".join(classes),
            "doubtfull" : cycle.doubtfull,
            "resolve_doubtfull" : resolved_doubtfull,
            "resolved_dates" : ",".join([date_to_string(x.start) for x in resolved_doubtfull]),
        }

    def _send(self, user, debut, status=None, message=None):
        fin = datetime.date(debut.year + 1, 1, 1)

        regles = list(Regle.objects.filter(user=user, date__gte=debut, date__lt=fin).order_by("date"))
        cycles = Cycles(user, start=debut, end=fin)
        if regles:
            start =  regles.pop(0)
            while regles:
                end = regles.pop(0)
                cycle = Cycle.create(user, start.date, end.date)
                cycles.append(cycle)
                start = end

        periods = cycles.get_periods(False)
        average = (sum(periods)/len(periods)) if periods else None
        return {
            "year": debut.year,
            "dates": reversed([self._get_cycle_data(i, x, average) for i, x in enumerate(cycles)]),
            "prev": debut.year - 1,
            "next": debut.year + 1,
            "count": len(cycles),
            "title": "Editer les cycles",
            "is_post" : message is not None,
            "message" : message,
            "status" : status
        }

    def _delete(self, user, date):
        debut = datetime.date(date.year, 1 , 1)
        message = f"La règle du {date_to_string(date)} a bien été supprimeé"
        status = True
        parsed_date = parse_date(date)
        regle=None
        try:
            regle = Regle.objects.get(user=user,  date=parsed_date)
        except Regle.DoesNotExist:
            message = f"Erreur aucune règle n'est enregistré au {date_to_string(date)}"
            status = False

        if regle is not None:
            try:
                regle.delete()
            except Exception as err:
                message = f"Impossible de supprimer la règle du {date_to_string(date)} : {err}"
                status = False

        return self._send(user, debut, status, message)

    def _repair(self, request):
        src_date = parse_date(request.POST["src"])
        new_dates = [parse_date(x) for x in request.POST["resolved"].split(",") if x]

        if not new_dates:
            return self._send(request.user, src_date, False, "Erreur aucune date de résolution n'a été trouvée")

        with transaction.atomic():
            try:
                regle = Regle.objects.get(user=request.user, date=src_date)
                regle.delete()
            except Regle.DoesNotExist:
                return self._send(request.user, src_date, False, f"Erreur impossible de trouver un cycle au {src_date}")
            except Exception:
                return self._send(request.user, src_date, False, f"Erreur de supprimer le cycle au {src_date}")

            for date in new_dates:
                try:
                    Regle.objects.create(user=request.user, date=date)
                except Exception:
                    return self._send(request.user, src_date, False, f"Erreur de créer le cycle au {date}")

        return  self._send(request.user, src_date, True, f"Le cycle a été modifié")







    def get_context_data(self, request, action_params, **kwargs):
        user = request.user
        debut = datetime.date.today()
        if request.method=="POST":
            return self._repair(request)
        elif "delete_date" in getattr(request, "GET", {}):
            return self._delete(user, parse_date(request.GET["delete_date"]))
        else:
            if "year" in getattr(request, "GET", {}):
                debut =datetime.date(int(request.GET["year"]), 1, 1)
            elif "year" in getattr(request, "POST", {}):
                debut = datetime.date(int(request.POST["year"]), 1, 1)
            else:
                debut = datetime.date(debut.year, 1 , 1)

            return self._send(user, debut)