import datetime

from djangotools.common.types import parse_date

from baby.app.context.base import BaseContextData, UserPrefences
from baby.app.models.const import get_min_period
from baby.app.models.regle import Regle


class ContextData(BaseContextData):
    page = "new_cycle"

    def ret(self, status=None, message=None):
        ret =  {
            "title" :  "Nouveau cycle"
        }
        if status is not None:
            ret.update({
                "status" : status,
                "message" : message,
                "is_post" : True
            })
        else:
            ret["is_post"] = False
        return ret
    def _post(self, request):
        user = request.user
        params = request.GET or request.POST
        force = params.get("force", False)
        date = params.get("date",datetime.date.today())
        if isinstance(date, (list, tuple)):
            if date: date = date[0]
            else: date=None
        today = parse_date(date)


        last = Regle.objects.last_regle_before(user, today)
        if last is not None:
            days_from_begin = (today - last.date).days

            if days_from_begin == 0:
                return self.ret(False, "Le cycle a déja été initialisé aujourd'hui")

            if days_from_begin < get_min_period(user) and not force:
                return self.ret(False, "Le cycle est trop court")
        try:
            Regle.objects.create(user, today)
        except Regle.DoesNotExist:
            return self.ret(False, "Le cycle a déja été initialisé aujourd'hui")

        return self.ret(True, "Le nouveau cycle a bien été pris en compte")

    def get_context_data(self, request, action_params, **kwargs):
        if request.method=="POST" or request.GET.get("validate") is not None :
            return self._post(request)
        else:
            return self.ret()


