from django.core.exceptions import ValidationError

from baby.app.context.base import BaseContextData, UserPrefences


def validate_choice(choices):
    def wrapper(data, choices=choices):
        choices = {k:k for k in choices}
        if data not in choices:
            raise ValidationError("Choix non autorisé")
        return choices[data]
    return wrapper

def validate_int(data):
    data = int(data)
    if 0 < data < 50: return data
    raise ValidationError("Valeur incorrecte")



class ContextData(BaseContextData):
    page = "parameters"
    types = {
        "ui.theme"  : validate_choice(["dark", "light"]),
        "regle.method"  : validate_choice(["contraception", "procreation"]),
        "regle.min_period" : validate_int,
        "regle.max_period" : validate_int,
        "regle.average_period_window" : validate_int,
    }
    def _do_one(self, data, key, value):
        validation = self.types[key]
        key =  key.split(".")
        for k in key[:-1]:
            if k not in data: return
            data = data[k]
        data[key[-1]] = validation(value)

    def _apply_update(self, request):
        args = request.POST
        main = request.user.pref.get("main")
        errors = {}
        for k, v in args.items():
            try:
                self._do_one(main, k, v)
            except ValidationError:
                errors[k.replace(".", "_")] = "Erreur de validation"
        if not errors:
            request.user.pref.save(main)
        return errors



    def get_context_data(self, request, action_params, **kwargs):
        errors = {}
        is_post = False
        if request.method == "POST":
            errors = self._apply_update(request)
            is_post = True

        user = request.user
        return {
            "title" :  "Paramètres",
            "errors" : errors,
            "is_post" : is_post,
            "status" : not errors
        }


