import datetime
import math

import pytest
from djangotools.common.types import parse_date, date_to_string

from baby.app.common.calendar import MonthCalendar
from baby.app.context.stats import ContextData as StatsContextData
from tests.util import create_regles

from tests.util import Request
@pytest.mark.django_db
class TestStats:
    def _req(self, lengthes, delta, get=None, date="01/10/2023"):
        if get is None:
            get = {}
        get.update({"today":  date_to_string(date)})
        return  Request("GET", create_regles(lengthes, delta, parse_date(date)), get=get)

    @pytest.mark.parametrize("content,offset", [
        ([24, 25, 26, 27], 1),
        ([24, 25, 26, 27, 27], 1),
        ([24, 25, 26, 27, 24], 1),
        ([24, 25, 26 ], 1),
    ])
    def test_nominal_case(self, client, content, offset):
        date = parse_date("01/10/2023")
        print(content, offset)
        req = self._req(content, offset, date=date)
        ret = req.test(StatsContextData)
        data = ret["context"]["data"]
        start = parse_date(data["start"])
        end = parse_date(data["end"])
        mini = data["min"]
        maxi = data["max"]
        average = data["average"]

        assert mini == min(content)
        assert maxi == max(content)
        assert average == sum(content) / len(content) if content else True
        assert start == date - datetime.timedelta(days=offset) if content else True
        assert end == start + datetime.timedelta(days=round(average))
        print(f"Average = {average}")


