import datetime
import math
from functools import partial

import pytest
from djangotools.common.types import parse_date, date_to_string

from baby.app.common.calendar import MonthCalendar
from baby.app.context.base import UserPrefences
from baby.app.context.edit import ContextData as EditContextData
from baby.app.models.regle import Cycle
from tests.util import create_regles, prefs

from tests.util import Request
@pytest.mark.django_db
class TestCcyles:

    def test_cycles_doubtfull_1(self):
        date = parse_date("01/10/2023")
        content = [25, 25, 75, 25, 25]
        offset = 0
        start = date - datetime.timedelta(days=sum(content)+offset)
        cycles = Cycle.from_date_range(create_regles(content, offset, date), start, date+datetime.timedelta(days=1))
        for i in range(len(content)):
            assert (content[i] > 40) == cycles[i].doubtfull
        no_doubt = cycles.resolve_doubtfull()

        assert all([not x.doubtfull for x in no_doubt])
        assert all([x.length == 25 for x in no_doubt])
        assert len(no_doubt) == 7

        return cycles

    def test_cycles_doubtfull_2(self):
        date = parse_date("01/10/2023")
        content = [25, 26, 27, 50, 24, 26, 28, 29]
        offset = 0
        start = date - datetime.timedelta(days=sum(content)+offset)
        cycles = Cycle.from_date_range(create_regles(content, offset, date), start, date+datetime.timedelta(days=1))
        for i in range(len(content)):
            assert (content[i] > 40) == cycles[i].doubtfull
        no_doubt = cycles.resolve_doubtfull()

        assert all([not x.doubtfull for x in no_doubt])
        assert len(no_doubt) == 9

        return cycles


    def test_cycles_doubtfull_3(self):
        date = parse_date("01/10/2023")
        content = [25, 50,]
        offset = 0
        start = date - datetime.timedelta(days=sum(content)+offset)
        cycles = Cycle.from_date_range(create_regles(content, offset, date), start, date+datetime.timedelta(days=1))
        for i in range(len(content)):
            assert (content[i] > 40) == cycles[i].doubtfull
        no_doubt = cycles.resolve_doubtfull()

        assert all([not x.doubtfull for x in no_doubt])
        assert len(no_doubt) == 3

        return cycles


    def test_cycles_doubtfull_4(self):
        date = parse_date("01/10/2023")
        content = [50, 25,]
        offset = 0
        start = date - datetime.timedelta(days=sum(content)+offset)
        cycles = Cycle.from_date_range(create_regles(content, offset, date), start, date+datetime.timedelta(days=1))
        for i in range(len(content)):
            assert (content[i] > 40) == cycles[i].doubtfull
        no_doubt = cycles.resolve_doubtfull()

        assert all([not x.doubtfull for x in no_doubt])
        assert len(no_doubt) == 3

        return cycles

    def test_cycles_doubtfull_5(self):
        date = parse_date("01/10/2023")
        content = [50,]
        offset = 0
        start = date - datetime.timedelta(days=sum(content)+offset)
        cycles = Cycle.from_date_range(create_regles(content, offset, date), start, date+datetime.timedelta(days=1))
        for i in range(len(content)):
            assert (content[i] > 40) == cycles[i].doubtfull
        no_doubt = cycles.resolve_doubtfull()

        assert all([not x.doubtfull for x in no_doubt])
        assert len(no_doubt) == 0
        return cycles

    def test_cycles_doubtfull_6(self):
        data = {'main': {"regle" : {
            "min_period" : 10,
            "max_period" : 38,
            "average_period_window" : 12, } }}
        date = parse_date("01/10/2023")

        content = [25, 25, 40, 24, 26, 27,]
        offset = 0
        start = date - datetime.timedelta(days=sum(content)+offset)
        cycles = Cycle.from_date_range(create_regles(content, offset, date, pref_class=prefs(data)),
                                       start, date+datetime.timedelta(days=1))
        for i in range(len(content)):
            assert (content[i] >= 40) == cycles[i].doubtfull
        no_doubt = cycles.resolve_doubtfull()

        assert all([not x.doubtfull for x in no_doubt])
        assert len(no_doubt) == 7
        return cycles


